@@ -5,7 +5,7 @@
 ;; Author: Jimmy Aguilar Mena
 ;; URL: https://github.com/Ergus/gtags-mode
 ;; Keywords: xref, project, imenu, gtags, global
-;; Version: 1.6
+;; Version: 1.8
 ;; Package-Requires: ((emacs "28"))
 
 ;; This program is free software: you can redistribute it and/or modify
@@ -62,11 +62,22 @@
   :type 'string
   :local t)
 
+(defcustom gtags-mode-update-args ""
+  "Extra arguments to use when updating the database."
+  :type 'string
+  :local t)
+
 (defcustom gtags-mode-lighter " Gtags"
   "The text displayed in the mode line."
   :type 'string
   :risky t)
 
+(defcustom gtags-mode-features '(project xref completion imenu hooks)
+  "The list of features enabled in gtags-mode.
+This variable must be set before enabling gtags-mode"
+  :type '(repeat symbol)
+  :risky t)
+
 (defcustom gtags-mode-verbose-level 2
   "The text displayed in the mode line."
   :type 'natnum
@@ -155,7 +166,7 @@ Start an asynchronous process and sets
 `gtags-mode--exec-async-sentinel' as the process sentinel.
 Returns the process object."
   (if-let* ((cmd (buffer-local-value cmd (current-buffer)))
-	    (command (append `(,cmd) args))
+	    (command (append `(,cmd) (string-split gtags-mode-update-args) args))
 	    (pr (make-process :name (format "%s-async" cmd)
 			      :buffer (generate-new-buffer " *temp*" t)
 			      :command command
@@ -193,7 +204,12 @@ On success return a list of strings or nil if any error occurred."
 
 (defun gtags-mode--create-plist (dir)
   "Return dbpath for DIR or nil if none."
-  (when-let* ((default-directory dir)
+  ;; Heuristic to create new plists only when visiting real files
+  ;; This optimizes when there is not tags file to avoid calling
+  ;; the external process repeatedly i.e in magit buffers that are
+  ;; regenerated every time and forgets the local variables
+  (when-let* (((and gtags-mode--global buffer-file-name))
+	      (default-directory dir)
 	      (root (car (gtags-mode--exec-sync "--print-dbpath"))))
     (setq root (concat (file-remote-p default-directory) ;; add remote prefix if remote
 		       (file-name-as-directory root)))   ;; add a / at the end is missing
@@ -201,24 +217,19 @@ On success return a list of strings or nil if any error occurred."
     (or (gtags-mode--get-plist root)   ;; already exist
 	(car (push `(:gtagsroot ,root :cache nil) gtags-mode--alist)))))
 
-(defun gtags-mode--set-local-plist (&optional dir)
+(defun gtags-mode--set-local-plist (dir)
   "Set and return the buffer local value of `gtags-mode--plist'."
-  (let ((default-directory (file-truename (or dir default-directory))))
+  (let ((default-directory (file-truename dir)))
     (gtags-mode--set-connection-locals)
     (setq-local gtags-mode--plist
 		(or (gtags-mode--get-plist default-directory)
-		    ;; Heuristic to create new plists only when visiting real files
-		    ;; This optimizes when there is not tags file to avoid calling
-		    ;; the external process repeatedly i.e in magit buffers that are
-		    ;; regenerated every time and forgets the local variables
-		    (when (buffer-file-name)
-		      (gtags-mode--create-plist default-directory))))))
+		    (gtags-mode--create-plist default-directory)))))
 
 (defun gtags-mode--local-plist (&optional dir)
   "Set and return the buffer local value of `gtags-mode--plist'."
   (if (local-variable-p 'gtags-mode--plist)
       gtags-mode--plist
-    (gtags-mode--set-local-plist dir)))
+    (gtags-mode--set-local-plist (or dir default-directory))))
 
 (defun gtags-mode--list-completions (prefix)
   "Get the list of completions for PREFIX.
@@ -249,7 +260,7 @@ name, code, file, line."
 					      line (1+ (match-beginning 1)) (match-end 1))) ;; file
 				(string-to-number (match-string-no-properties 3 line))))) ;; line
 		   (apply #'gtags-mode--exec-sync
-		    (append args gtags-mode--output-format-options `(,symbol)) )))
+		    (append gtags-mode--output-format-options args `(,symbol)) )))
     (error "Calling gtags-mode--filter-find-symbol without GTAGSROOT")
     nil))
 
@@ -288,7 +299,7 @@ to set it.  This is needed when saving new created files because they
 won't have `buffer-file-name' but will just acquire one."
   (when (and buffer-file-name
 	     (or gtags-mode--plist
-		 (gtags-mode--set-local-plist buffer-file-name)))
+		 (gtags-mode--set-local-plist default-directory)))
     (gtags-mode--exec-async
      'gtags-mode--global
      "--single-update" (file-name-nondirectory buffer-file-name))))
@@ -312,7 +323,7 @@ Return as a list of xref location objects."
 
 (cl-defmethod xref-backend-references ((_backend (head :gtagsroot)) symbol)
   "List all referenced for SYMBOL."
-  (gtags-mode--xref-find-symbol '("--reference") symbol))
+  (gtags-mode--xref-find-symbol '("--reference" "--symbol") symbol))
 
 (cl-defmethod xref-backend-apropos ((_backend (head :gtagsroot)) symbol)
   "List grepped list of candidates SYMBOL."
@@ -374,6 +385,11 @@ Return as a list of xref location objects."
 	      (completion-table-dynamic #'gtags-mode--list-completions)
 	      :exclusive 'no))))
 
+(defmacro gtags-mode--with-feature (feature &rest body)
+  (declare (indent 1) (debug t))
+  `(when (memq ,feature gtags-mode-features)
+     ,@body))
+
 ;;;###autoload
 (define-minor-mode gtags-mode
   "Use GNU Global as backend for project, xref, capf and imenu.
@@ -383,11 +399,16 @@ rely on their original or user configured default behavior."
   :lighter gtags-mode-lighter
   (cond
    (gtags-mode
-    (add-hook 'project-find-functions #'gtags-mode--local-plist)
-    (add-hook 'xref-backend-functions #'gtags-mode--local-plist)
-    (add-hook 'completion-at-point-functions #'gtags-mode-completion-function)
-    (add-hook 'after-save-hook #'gtags-mode--after-save-hook)
-    (advice-add imenu-create-index-function :before-until #'gtags-mode--imenu-advice))
+    (gtags-mode--with-feature 'project
+      (add-hook 'project-find-functions #'gtags-mode--local-plist))
+    (gtags-mode--with-feature 'xref
+      (add-hook 'xref-backend-functions #'gtags-mode--local-plist))
+    (gtags-mode--with-feature 'completion
+      (add-hook 'completion-at-point-functions #'gtags-mode-completion-function))
+    (gtags-mode--with-feature 'hooks
+      (add-hook 'after-save-hook #'gtags-mode--after-save-hook))
+    (gtags-mode--with-feature 'imenu
+      (advice-add imenu-create-index-function :before-until #'gtags-mode--imenu-advice)))
    (t
     (remove-hook 'project-find-functions #'gtags-mode--local-plist)
     (remove-hook 'xref-backend-functions #'gtags-mode--local-plist)
